﻿#region Copyright
/// <summary>The ScummVM WPF UI project</summary>
/// <copyright>(c) 2016 MojoTouch</copyright>
/// <author>Ilya Rovensky</author>
#endregion

using System;
using System.Drawing;
using System.Runtime.InteropServices;
using System.Windows;
using System.Windows.Interop;

namespace WpfInterface
{
    public partial class UIWindow : Window //NavigationWindow
    {
        readonly Rectangle screenRectangle;

        public UIWindow(System.Windows.Forms.Screen screen)
        {
            screenRectangle = screen.WorkingArea;
            InitializeComponent();
        }

        [DllImport("user32.dll", SetLastError = true)]
        static extern bool MoveWindow(IntPtr hWnd, int X, int Y, int nWidth, int nHeight, bool bRepaint);

        protected override void OnSourceInitialized(EventArgs e)
        {
            base.OnSourceInitialized(e);
            var wih = new WindowInteropHelper(this);
            IntPtr hWnd = wih.Handle;
            MoveWindow(hWnd, screenRectangle.Left, screenRectangle.Top, screenRectangle.Width, screenRectangle.Height, false);
        }

        void Window_Loaded(object sender, RoutedEventArgs e)
        {
            WindowState = WindowState.Maximized;
        }

        //Window _emptyPage = new Window();
        //private System.Timers.Timer _timer;
        private bool _isClosing;

        public UIWindow()
        {
            InitializeComponent();
            
            //_timer = new System.Timers.Timer(200)
            //{
            //    AutoReset = false,
            //    Enabled = true
            //};

            //_timer.Elapsed += OnTimer;
        }

        //private void OnTimer(object sender, ElapsedEventArgs e)
        //{
        //    if (UserInterface.OglWindowHandle != IntPtr.Zero)
        //    {
        //        Win32Window.Rect rectangle = new Win32Window.Rect();
        //        if (Win32Window.GetWindowRect(UserInterface.OglWindowHandle, ref rectangle))
        //        {
        //            Dispatcher.BeginInvoke(new Action(() =>
        //            {
        //                WindowState = WindowState.Normal;
        //                Left = rectangle.Left;
        //                Top = rectangle.Top;
        //                Width = rectangle.Right - rectangle.Left;
        //                Height = rectangle.Bottom - rectangle.Top;
        //            }));
        //        }

        //        //if (!_removedFromAltTab)
        //        //{
        //        //    Dispatcher.BeginInvoke(new Action(() => EnableAltTab(false)));
        //        //    _removedFromAltTab = true;
        //        //}
        //    }
        //}        

        private Menu _currentMenu;
        public Menu CurrentMenu
        {
            get
            {
                return _currentMenu;
                //return Content is Menu ? (Menu)Content : null;
            }
            set
            {
                //Menu prevMenu = _currentMenu;
                
                Content = null;
                _currentMenu = value;

                if (value != null)
                {
                    Content = value;
                    Activate();
                }

                //if (prevMenu != null)
                //    prevMenu.Visibility = Visibility.Hidden;

                if (!_isClosing)
                    Win32Window.EnableAltTab(this, Content != null);
            }
        }

        private void NavigationWindow_Closed(object sender, EventArgs e)
        {
            //MessageBox.Show("window is closing");
            // Sends WM_CLOSE message to the OpenGL window
            //if (UserInterface.OglWindowHandle != IntPtr.Zero)
            //    Win32Window.SendMessage(UserInterface.OglWindowHandle, 0x10, IntPtr.Zero, IntPtr.Zero);
            //Application.Current.Shutdown();
        }

        private void Window_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            _isClosing = true;
            if (_currentMenu != null)
                _currentMenu.Result = MenuAction.Quit;
        }
    }
}
